class Colors:
	background = '#2d2d2d'
	aux = '#585869'
	text = '#ffffff'
	piano_track = '#bdd0df'
	graph_background = aux # '#85c9ac'
	scroll_trough = 'lightgray' # '#daebe3'
	graph_foreground = 'white'
	score_label = '#60e0a9'
	red = '#ab1302'
	green = '#30bf13'
	yellow = '#ffd324'
	tuner_needle = '#6b3e32' # '#452A23'
	settings_color = '#b6aecf'